package edu.udo.cs.ie.cowbots.agent;

import java.util.*;
import java.util.Map.Entry;
import java.util.logging.*;

import jason.JasonException;
import jason.RevisionFailedException;
import jason.asSyntax.*;
import jason.asSyntax.PlanBody.BodyType;
import jason.asSyntax.Trigger.TEOperator;
import jason.asSyntax.Trigger.TEType;
import jason.asSyntax.parser.ParseException;
import jason.asSemantics.*;
import jason.asSemantics.TransitionSystem.State;
import jason.bb.BeliefBase;
import jason.runtime.*;
import jason.stdlib.foreach;
import jason.stdlib.loop;
import edu.udo.cs.ie.cowbots.bdi.*;
import edu.udo.cs.ie.cowbots.bdi.actsel.CActionComponent;
import edu.udo.cs.ie.cowbots.bdi.actsel.IActionComponent;
import edu.udo.cs.ie.cowbots.bdi.delib.DeliberationComponent;
import edu.udo.cs.ie.cowbots.bdi.desgen.CDesire;
import edu.udo.cs.ie.cowbots.bdi.desgen.DesireComponent;
import edu.udo.cs.ie.cowbots.bdi.desgen.FastCowbotDesireComponent;
import edu.udo.cs.ie.cowbots.bdi.brf.*;
import edu.udo.cs.ie.cowbots.internalActions.experimental2.exRevision;
import edu.udo.cs.ie.cowbots.launcher.CowbotsConfig;
import edu.udo.cs.ie.cowbots.logic.ELP;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.logic.solver.*;
import edu.udo.cs.ie.cowbots.ui.*;
import edu.udo.cs.ie.cowbots.util.GridMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.*;

import java.io.*;

/**
 * cowbots customisation of jason's agent class. this class integrates the
 * cowbot bdi extensions into jason.
 * 
 * @author Thomas
 * 
 */
public class CowbotAgent extends Agent implements ICowbot, CircumstanceListener {
	// these are the variables for our
	// bdi implementation
	CRevisionComponent cowRevision = null;
	DesireComponent cowDesGen = null;
	DeliberationComponent cowDelib = null;
	IActionComponent cowAction = null;
	ICowbotUI cowUI = null;
	List<Literal> initalKnowledge = null;


	// variable to a dlv instance
	DLV dlv = null;

	// path variables
	String agTemp = null;
	String agDir = null;

	// name of agent
	String agName = null;

	// agent logger, intention stack and directory
	Logger log;
	Logger intLog;
	String logDir;
	
	// hotfix variables
	// used during development only
	boolean fix_GenerateBeliefChangeEvents = false;
	boolean fix_UseDefaultSelectIntention = true;
	boolean fix_UseDefaultSelectEvent = false;
	boolean fix_UseDefaultSelectOption = true;
	boolean fix_MessageEvent = false;
	boolean fix_NoMotivation = false;
	
	// process cowbot message at socAcc (true)
	// or selectMessage (false). default is
	// false, can be set via the local_config.xml
	boolean optProcessMessageInSocAcc = false;
	
	// if true, deliberation function is not called
	// whenever an intention finished.
	boolean optDelibNoIntentionRestart = false;
	
	// if true, extended error messages are printed
	boolean optAllInfo = false;
		
	// system time, set by the agent architecture
	int agTime = 0;
	
	// enable detailed logging
	boolean loggingEnabled = false;
	
	// some configuration variables, used for finetuning
	// an agent during startup
	List<String>				agconfPrograms,agconfNoRevisePrograms;
	HashMap<String,String>		agconfUserPrograms;
	boolean						agconfNoDelib,agconfMSGsocAcc;
	String						agconfBRFCLASS;

	// helper class to hold simulation data
	class SimState {
		public	int		simID;
		public	int		time;
		public	int		turns;
		public	int		cMinX,cMaxX,cMinY,cMaxY;
		public	int		width, height;
		
		
		SimState(List<Literal> percepts) {
			// 0 is the simulation id predicate
			// 1 is the world size		
			// 2 is the corral position
			Literal lSim = percepts.get(0);
			Literal lWS = percepts.get(1);
			Literal lCS = percepts.get(6);
			
			width = Integer.parseInt(lWS.getTerm(0).toString());
			height = Integer.parseInt(lWS.getTerm(1).toString());
			
			cMinX = Integer.parseInt(lCS.getTerm(0).toString());
			cMinY = Integer.parseInt(lCS.getTerm(1).toString());
			cMaxX = Integer.parseInt(lCS.getTerm(2).toString());
			cMaxY = Integer.parseInt(lCS.getTerm(3).toString());
			
			time = 0;
			turns = 0;
						
		}
		
		public boolean needReset(SimState other) {
			if (simID != other.simID)
				return true;
			if ((cMinX != other.cMinX) ||
				(cMinY != other.cMinY) ||
				(cMaxX != other.cMaxX) ||
				(cMaxY != other.cMaxY))
				return true;
			if (width != other.width)
				return true;
			if (height != other.height)
				return true;
			
			return false;
		}
	}
	
	// track simulation state
	boolean		agstateReset;
	SimState	simstate = null;
	
	/**
	 * this method initializes the cowbot extensions. the
	 * configuration options are taken from the user settings
	 * in the mas2j file. 
	 * 
	 * @param stts user settings
	 */
	public void initBDI(Settings stts, String agName) throws Exception {

		agconfPrograms = new LinkedList<String>();
		agconfNoRevisePrograms = new LinkedList<String>();
		agconfUserPrograms = new HashMap<String,String>();
		
		// create agent logger
		this.agName = agName;
		logDir = "log";
		try {
			File f = new File("log");
			f.mkdir();

			FileHandler fh = new FileHandler(logDir + "/" + agName + ".log",
					false);
			fh.setFormatter(new SimpleFormatter());
			log = Logger.getLogger(agName);
			log.setUseParentHandlers(false);
			log.addHandler(fh);
			log.info("agent initializing");
			
			fh = new FileHandler(logDir+"/" + agName+".intention.log",false);
			fh.setFormatter(new SimpleFormatter());
			intLog = Logger.getLogger(agName+".intentions");
			intLog.setUseParentHandlers(false);
			intLog.addHandler(fh);
		} catch (Exception e) {

		}
		
		logDir += "/";
		
		File f = new File("");
		this.agDir = f.getAbsolutePath() + "/";
		try {
			f = new File("temp");
			f.mkdir();
		} catch (Exception e) {

		}
		
		try {
			f = new File(this.agDir + "temp/" + agName + "/");
			f.mkdir();
			this.agTemp = f.getAbsolutePath() + "/";
			this.agTemp = this.agTemp.replaceAll("\\\\", "/");

		} catch (Exception e) {
			System.out.println(e);
			System.out.println("error trace=" + e.getStackTrace());
		}

		CowbotsConfig cbcfg = CowbotsConfig.getInstance();
		
		String option = cbcfg.getOption("deliberation", "norestart");
		if (option != null)
			optDelibNoIntentionRestart = option.equalsIgnoreCase("true");
		option = cbcfg.getOption("agclass", "msgproc");
		if (option != null)
			optProcessMessageInSocAcc = option.equalsIgnoreCase("socAcc");
		
		// load dlv path
		String dlvpath = null;
		if (stts != null) {
			// look where dlv is
			dlvpath = stts.getUserParameter("dlv");
			
			if (dlvpath != null) {
				dlvpath = dlvpath.replaceAll("\"", "");				
			}
		}
		
		// config override
		if (cbcfg.getDLVPath() != null) {
			dlvpath = cbcfg.getDLVPath();
			stts.getUserParameters().put("dlv", dlvpath);
		}
		
		dlv = new DLV(dlvpath);
		dlv.setLogPath(logDir);

		// setup pg545 extensions
		// create ui, unless disabled or gui creation fails
		// required to run the cowbots mas on computers
		// without a graphics device 
		String gui = stts.getUserParameter("gui");
		if (gui != null) {
			if (gui.equalsIgnoreCase("disabled"))
				cowUI = new DummyUI();
		}
		if (cowUI == null) {
			try {
				cowUI = CowUI.createUI(this.getTS().getUserAgArch().getAgName());
			} catch (Exception e) {
				cowUI = new DummyUI();
			}
		}

		String agConfigFile = stts.getUserParameter("agcfg");
		if (agConfigFile != null) {
			agConfigFile = agConfigFile.replaceAll("\"", "");
			agConfigFile = agConfigFile.trim();
		} else {
			agConfigFile = "agents.xml";
		}
		
		// read agent configuration file 
		initAgFromXML(agConfigFile,this.agName);

		
		// Hotfix: Using fast, java-based Desire Component instead...
		// cowDesGen = new DesireComponent(stts);
		cowDesGen = new FastCowbotDesireComponent(cowUI);
		cowAction = new CActionComponent();
		
		// check if we want to use deliberation
		if (!this.agconfNoDelib)
			cowDelib = new DeliberationComponent(this.agName);
		else
			cowDelib = null;
				
		// choose revision class
		try {
			if (this.agconfBRFCLASS != null) {
				this.cowRevision = (CRevisionComponent) Class.forName(this.agconfBRFCLASS).newInstance();
				System.out.println(this.agName+": using revision class: "+this.agconfBRFCLASS);
			}
			else
				this.cowRevision = new CowbotRevision();
		} catch (Exception e) {
			System.err.println("FATAL! could not load revision class: "+this.agconfBRFCLASS);
			System.exit(0);
		}
		//cowRevision = new CowbotRevision();
		//cowRevision = new exRevision();
		cowRevision.init(stts, dlv, cowUI);
		
		for (String pname : this.agconfPrograms) {
			ELP elp = ELP.loadFrom(pname);
			cowRevision.addProgram(elp, this.agName);			
		}
		for (String pname : this.agconfNoRevisePrograms) {
			ELP elp = ELP.loadFrom(pname);
			this.cowRevision.getBeliefBase().addBeliefSetProgram(elp);			
		}
		for (Entry<String,String> pe : this.agconfUserPrograms.entrySet()) {
			ELP elp = ELP.loadFrom(pe.getValue());
			cowRevision.addUserProgram(pe.getKey(), elp);			
		}
		
		
		// add important initial beliefs to epistemic state
		initalKnowledge = new LinkedList<Literal>();
		Iterator<Literal> initialBels = this.bb.iterator();
		while (initialBels.hasNext()) {
			Literal lit = initialBels.next();
			initalKnowledge.add(lit.copy());
			this.cowRevision.revise_belief(lit, null, null);
		}
		Literal iam = Literal.parseLiteral("iam(" + agName + ")");
		
		initalKnowledge.add(iam);
		
		this.cowRevision.revise_belief(iam, null, null);
		this.bb.add(iam);
		
		this.cowDesGen.updateDesireState(this.cowRevision.getEpistemicState());				
		
		// if we have a deliberation, use the listener and set k-factor
		// (after a year still nobody knows what it is good for) 
		if (this.cowDelib != null) {
			this.ts.getC().addEventListener(this);
			cowDelib.setK(1.3);
		}
					
		// first simulation we take part can be done with
		// initial configuration
		agstateReset = true;
	}

	/**
	 * this methods loads an agents configuration from an xml file
	 * 
	 * @param string
	 * @param agName2
	 */
	private void initAgFromXML(String filename, String agName) {
			
		Element agConfig = null;
		Element personal_config = null;
		Document doc = null;
		NodeList nl;
		Element elem;
		
		try {
			
			
			DocumentBuilder docbld = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			doc = docbld.parse(new File(filename));			
			nl = doc.getElementsByTagName("agent");
			
			for (int i = 0; i < nl.getLength(); i++) {
				elem = (Element) nl.item(i);
				if ( elem.getAttribute("default").equalsIgnoreCase("yes") ) {
					agConfig = elem;
				}
				if ( elem.getAttribute("name").equalsIgnoreCase(agName)) {
					personal_config = elem;
				}
			}
						
			docbld = null;
			
		} catch (Exception e) {
			System.err.println(this.agName+": could not load agent configuration file: "+filename);
		}
		
		if (personal_config != null)
			agConfig = personal_config;
		
		if (agConfig == null) {
			this.log.info("no configuration for agent "+agName+" found in file "+filename+"!" );
			return;
		}
		
		// get all programs to load
		nl = agConfig.getElementsByTagName("belief");
		for (int i = 0; i < nl.getLength(); i++) {
			elem = (Element) nl.item(i);
			String src = elem.getAttribute("src").trim();
			if (src.length() < 1)
				continue;
			
			// do we have a belief set processing program?
			boolean bsetprogram = elem.getAttribute("norevise").trim().equalsIgnoreCase("true");
			ELP prg = ELP.loadFrom(src);
			if (prg == null) {
				log.info("loading program "+src+" failed!");
			} else {				
				log.info("loading program "+src+":\n"+prg.toString());
				if (!bsetprogram)
					//this.cowRevision.addProgram(prg, this.agName);
					this.agconfPrograms.add(src);
				else
					//this.cowRevision.getBeliefBase().addBeliefSetProgram(prg);
					this.agconfNoRevisePrograms.add(src);
			}
		}
		
		// get all programs to load
		nl = agConfig.getElementsByTagName("program");
		for (int i = 0; i < nl.getLength(); i++) {
			elem = (Element) nl.item(i);
			String src = elem.getAttribute("src").trim();
			if (src.length() < 1)
				continue;
			
			String name = elem.getAttribute("name").trim();
			if (name.length() < 1)
				continue;
			
			ELP prg = ELP.loadFrom(src);
			if (prg == null) {
				log.info("loading user program "+src+" failed!");
			} else {				
				log.info("loading user program "+src+":\n"+prg.toString());
				//this.cowRevision.addProgram(prg, this.agName);
				this.agconfPrograms.add(src);
			}
		}
		
		// get all config elements
		nl = agConfig.getElementsByTagName("config");
		for (int i = 0; i < nl.getLength(); i++) {
			elem = (Element) nl.item(i);
			String param = elem.getAttribute("parameter").trim();
			String value = elem.getAttribute("value").trim();
			
			// look for deliberation options
			if (param.equalsIgnoreCase("deliberation")) {
				System.out.println(this.agName+": what to do with delib: "+value);
				if (value.equalsIgnoreCase("disable")) {
					//this.cowDelib = null;
					this.agconfNoDelib = true;
					optDelibNoIntentionRestart = true;
				}
			} else if (param.equalsIgnoreCase("msgProc")) {
				if (value.equalsIgnoreCase("socAcc")) {
					this.optProcessMessageInSocAcc = true;
					this.agconfMSGsocAcc = true;
					System.out.println(this.agName+": handling messages in socAcc");
				}
			}  else if (param.equalsIgnoreCase("BRFCLASS")) {
				this.agconfBRFCLASS = value;
			}
		}
		
		doc = null;
	}

	/**
	 * belief update function, called whenever an agent perceives information
	 * about its environment. whenever new perceptions are avail, the
	 * cowbotrevision fetches the environmental update, and the desire
	 * generation is invoked to update an agent's desires. at last, a dummy
	 * event is fired to force the jason interpreter to step into the event
	 * selection, where deliberation takes place.
	 * 
	 */
	@Override
	public void buf(List<Literal> percepts) {
		// if nothing new to do..
		if (percepts == null)
			return;
		
		if (percepts.size() == 0)
			return;
		
		// handle special events send by the environment
		onNewPercepts(percepts);
		
		// might be cleared, so return
		if (percepts.size() == 0)
			return;			
		
		// if we have a simulation, start at step 2 instead of one,
		// this gives an agent time to adapt map changes
		if (this.simstate != null)
			if (this.simstate.time == 1)
				return;
		
		long brfTime = System.nanoTime();
		
		// 1: call the revision method to integrate new beliefs		
		try {
			cowRevision.revise_percepts(percepts, null);
			this.agTime = cowRevision.getEpistemicState().getWorldTime();
		} catch (Exception e) {
			System.out.println("brf fehler: "+e);
			e.printStackTrace();
		}		
		
		// 1b: update jason bb (difference update)
		this.syncJasonBB(false);
		
		// fix: if world time is zero, break here
		if (this.agTime < 1)
			return;
		
		List<ELPLiteral> exploredCells = cowRevision.getEpistemicState().getExploredCells();
		// 1c: check time, broadcast stuff at t>0
		if ((cowRevision.getEpistemicState().getWorldTime() > 0) && 
			(exploredCells.size() > 0)) {
				ts.updateEvents( new Event( new Trigger(TEOperator.add, TEType.achieve, Literal.parseLiteral("broadcastExploredCells")), Intention.EmptyInt) );			
			} else {
				//System.out.println("could not broadcast cows: "+ exploredCells.size());
				// no new cells!
				ts.updateEvents( new Event( new Trigger(TEOperator.add, TEType.achieve, Literal.parseLiteral("broadcastExploredCells")), Intention.EmptyInt) );
			}
				
		brfTime = System.nanoTime() - brfTime;


		
		// 2: call motivation component
		long desgenTime = System.nanoTime();
		
		try {
			cowDesGen.updateDesireState(cowRevision.getEpistemicState());
			this.cowUI.updateDesireState(cowDesGen.getDesireState());			
		} catch (Exception e) {
			System.out.println("desgen fehler: "+e);
			e.printStackTrace();
		}
				
		desgenTime = System.nanoTime() - desgenTime;
		
		long delibTime = System.nanoTime();		

		if (this.cowDelib != null)
			runDelib();
		else {
			// take the top motivated desire and fire it as an event
			CDesire des0 = this.cowDesGen.getDesireState().getMostMotivatedDesire();
			this.ts.updateEvents( new Event(Trigger.parseTrigger("+!"+des0.getUniqueName()), Intention.EmptyInt) );
			log.info("buf: choosing desire "+des0.toString());
		}
		
		delibTime = System.nanoTime() - delibTime;
		
		brfTime /= 1000000;
		desgenTime /= 1000000;
		delibTime /= 1000000;
		log.info("buf zeiten (brf,desgen,delib): " + brfTime + "ms, " + desgenTime + "ms, " + delibTime + "ms");
	}
	
	
	/**
	 * main handler for handling percepts send from an environment 
	 * @param percepts
	 */
	protected void onNewPercepts(List<Literal> percepts) {
		Literal l0 = percepts.get(0);
		if (l0.getFunctor().equalsIgnoreCase("simulation"))
			onSimulationStart(percepts);
		if (l0.getFunctor().equals("endOfSimulation")) {
			onSimulationEnd(percepts);
			percepts.clear();
		}
		if (l0.getFunctor().equals("time")) {
			if (this.simstate != null)
				this.simstate.time = Integer.parseInt( l0.getTerm(0).toString() );
		}
	}

	
	/**
	 * this method is used to setup an agent for a simulation. during
	 * lifetime of an agent, he may assume that each simulation id
	 * is unique. so receiving the same simulation multiple times,
	 * he should not reinitialize himself again.
	 * @param percepts
	 */
	protected void onSimulationStart(List<Literal> percepts) {
		
		
		SimState sState = new SimState(percepts);
		percepts.remove( percepts.size()-1);
		
		// no need to reset, first simulation
		if (this.simstate == null) {
			this.simstate = sState;
		}
		
		if (agstateReset || (this.simstate.needReset(sState))) {
			GridMap gm = this.cowRevision.getEpistemicState().getMap();
			gm.setDimensionContest(sState.width, sState.height, sState.cMinX, sState.cMinY, sState.cMaxX, sState.cMaxY);
			this.cowRevision.reset();
			this.agstateReset = false;
			this.simstate = sState;
			
			// clear bb		
			Iterator<Literal> lIter = this.bb.iterator();
			List<Literal> toPrune = new LinkedList<Literal>();
			while (lIter.hasNext()) {
				toPrune.add(lIter.next());			
			}
			lIter = toPrune.iterator();
			while (lIter.hasNext())
				this.bb.remove(lIter.next());
			
			lIter = this.initalKnowledge.iterator();
			while (lIter.hasNext()) {
				Literal lit = lIter.next();
				this.bb.add(lit.copy());	// need to add copy to not mess up things with sources
				this.cowRevision.revise_belief(lit, null, null);
			}
			
			// create a new desire state
			this.cowDesGen = new FastCowbotDesireComponent(cowUI);
		
			Queue<Message> mq = this.ts.getC().getMailBox();
			if (mq != null)
				mq.clear();
					
			// do not clear percepts here			
		}
	}
	
	/**
	 * this method handles end of simulation events. the agent is put
	 * into a reset-state and won't accept new messages unless it
	 * receives a simulation start event.
	 * 
	 * @param per	percepts containing the end of simulation data
	 */
	protected void onSimulationEnd(List<Literal> per) {
		// reset the agent
		System.out.println("end of simulation, resetting agent");
		agstateReset = true;		
		this.ts.getC().clearEvents();
		this.ts.getC().clearIntentions();
		this.ts.getC().clearPendingActions();
		this.ts.getC().clearPendingIntentions();
		Queue<Message> mq = this.ts.getC().getMailBox();
		if (mq != null)
			mq.clear();
		per.clear();
	}

	
	/**
	 * belief revision function. called by jason for belief addtion
	 * or deletion.
	 * 
	 */
	@Override
	public List<Literal>[] brf(Literal beliefToAdd, Literal beliefToDel,
			Intention i) throws RevisionFailedException {
		if (cowRevision == null)
			return super.brf(beliefToAdd, beliefToDel, i);

		 List<Literal>[] result = null;
		try {
			if (beliefToAdd != null) {
				if (logger.isLoggable(Level.FINE))
					logger.fine("Adding belief " + beliefToAdd);

				if (getBB().add(beliefToAdd)) {
					result = new List[2];
					result[0] = Collections.singletonList(beliefToAdd);
					result[1] = Collections.emptyList();
					this.cowRevision.revise_belief(beliefToAdd, null, null);
				} else {
					if (this.optAllInfo)
						System.out.println(this.agName+" could not add belief: "+beliefToAdd);
				}
			}

			if (beliefToDel != null) {
				Unifier u = null;
				try {
					u = i.peek().getUnif(); // get from current intention
				} catch (Exception e) {
					u = new Unifier();
				}

				if (logger.isLoggable(Level.FINE))
					logger.fine("Doing brf for " + beliefToDel + " in BB="
							+ believes(beliefToDel, u));

				if (believes(beliefToDel, u)) {
					beliefToDel.apply(u);
					if (getBB().remove(beliefToDel)) {
						if (logger.isLoggable(Level.FINE))
							logger.fine("Removed:" + beliefToDel);
						if (result == null) {
							result = new List[2];
							result[0] = Collections.emptyList();
						}
						result[1] = Collections.singletonList(beliefToDel);
						this.cowRevision.revise_belief(null, beliefToDel, null);
					} else {
						if (this.optAllInfo)
							System.out.println(this.agName+" could not delete belief: "+beliefToDel);
					}
				} else {
					if (this.optAllInfo)
						System.out.println(this.agName+" could not unify belief to del: "+beliefToDel);
				}

			}
		} catch (Exception e) {
			logger.log(Level.WARNING, "Error at BRF.", e);
		}
				
		return result;
	}

	/**
	 * this method synchronizes the logical belief set with
	 * jason's belief base.
	 */
	protected void syncJasonBB(boolean createEvents) {
		List<Literal>[] belChange = cowRevision.getChangeOfBeliefs();
		for (Literal l : belChange[1]) {
			if (!l.hasSource())
				l.addAnnot(BeliefBase.TPercept);
			this.bb.remove(l);
		}
		for (Literal l : belChange[0]) {
			if (!l.hasSource())
				l.addAnnot(BeliefBase.TPercept);
			
			if (!this.bb.add(l)) {
				
			} else {
				if (createEvents)
					ts.updateEvents( new Event( new Trigger(TEOperator.add, TEType.belief, l), Intention.EmptyInt) );
			}
		}
	}
	

	@Override
	public Event selectEvent(Queue<Event> events) {
		if ((cowDelib != null) && (!fix_UseDefaultSelectEvent)) {
			if (this.optDelibNoIntentionRestart)
				return super.selectEvent(events);
			else
				return cowDelib.reconsider(events,cowDesGen.getDesireState(),this.getTS());
		} else
			return super.selectEvent(events);
	}

	@Override
	public Option selectOption(List<Option> options) {
		if ((cowDelib != null) && (!fix_UseDefaultSelectOption))
			return cowDelib.choose(options);
		else
			return super.selectOption(options);
	}

	@Override
	@SuppressWarnings("unchecked")
	// TODO: testen
	public Intention selectIntention(Queue<Intention> intentions) {

		if ((cowDelib != null) && (!fix_UseDefaultSelectIntention)) {
			Intention selectedIntention = cowDelib.select(intentions);
			return selectedIntention;
		} else
			return super.selectIntention(intentions);
	}

	public IEpistemicState getEpistemicState() {
		return this.cowRevision.getEpistemicState();
	}
	
	
	/**
	 * this method is used to let an agent decide if it accepts
	 * or rejects an incoming message from other agents. in our
	 * implementation, we accept all messages by default. if
	 * the message contains a specific header (first literal is
	 * messagecontent/1), then the message is stored into the
	 * elp belief base. otherwise, the message is passed down
	 * to jason's interpreter for further processing.
	 * 
	 */
	@Override
	public boolean socAcc(Message m) {
		// if the revision can handle this message,
		// let it process it and return false.
		// otherwise, accept message
	
		if (optProcessMessageInSocAcc) {
			
			// only process messages if an agent is ready
			// otherwise, bad things might happen
			if (this.simstate == null)
				return false;			
			
			// agent is in reset mode, do not accept messages
			if (this.agstateReset)
				return false;
			
			this.cowRevision.queueMessage(m);
			this.cowRevision.processMessages(this.agName);
			Queue<Message> q = this.cowRevision.getEpistemicState().getMessages();
			if (q.size() == 0)
				return false;
			else {
				q.clear();
				return true;
			}
		} else
			return true;
	}
	
	/**
	 * this method selects a message from an agent's message
	 * queue and return it to the transition system. in our
	 * implementation. it additionally checks any message if
	 * it represents a message with a logical program in that
	 * case, the message is handled by the revision system.
	 */
	@Override
	public Message selectMessage(Queue<Message> messages) {
		if (this.optProcessMessageInSocAcc) {
			return super.selectMessage(messages);
		} else {
					
			while (messages.peek() != null) {
				Message m = messages.poll();
				this.cowRevision.queueMessage(m);
			}
	
			// process messages
			this.cowRevision.processMessages(this.agName);
						
			// generate events here
			this.syncJasonBB(true);
			
			// let deliberation send messages to jason
			if (this.cowDelib != null) {
				this.cowDelib.reconsiderMessage(this.cowRevision
						.getEpistemicState(), this.getTS(), this.agName);
				
				// call other components to adapt changes
				// comment this call out if desire generation causes
				// too much performance problems
				this.cowDesGen.updateDesireState(this.cowRevision.getEpistemicState());
													
				// let deliberation run
				runDelib();
				return null;
			} else {
				Queue<Message> mq = this.cowRevision.getEpistemicState().getMessages();
				if (!mq.isEmpty()) {
					Message ret = mq.poll();
					messages.addAll(mq);
					return ret;
				} else
					return null;					 
			}
		}
	}

	/**
	 * @return the cowDelib
	 */
	public DeliberationComponent getCowDelib() {
		return this.cowDelib;
	}

	@Override
	public DLV getDLV() {
		return this.dlv;
	}

	@Override
	public IActionComponent getActionSelection() {
		return this.cowAction;
	}

	@Override
	public String getLogDirectory() {
		return this.logDir;
	}

	@Override
	public String getTemporaryDirectory() {
		return this.agTemp;
	}

	@Override
	public ICowbotUI getUI() {
		return this.cowUI;
	}

	@Override
	public void eventAdded(Event arg0) {		
	}

	@Override
	public void intentionAdded(Intention arg0) {
		
		if (loggingEnabled) {
			if (arg0 == null) {
				intLog.info(this.agName+" warning! null as intention added to the TS");
				return;
			}
			if (arg0.getIMs().size()==0) {
				// ok, empty intention added
				return;
			}
			String int0 = this.agName+" intention added: "+arg0.getId()+
			" at world time " + this.agTime + "\n";
			Unifier unif = arg0.peek().getUnif();
			for (int i = arg0.getIMs().size()-1; i >= 0;i--) {
				// print intended mean (trigger)
				IntendedMeans im = arg0.getIMs().get(i);
				int0 += "IM "+i+": trigger: "+im.getTrigger().toString() + "";
				// context
				//int0 += im.getUnif().toString()+"\n"; 
				Plan pl = im.getPlan();
				if (pl != null) {
					LogicalFormula lf = pl.getContext();
					if ((lf != null) && (unif != null)) {
						lf = (LogicalFormula) lf.clone();
						
						lf.apply( unif );
						int0 += " : " +lf.toString();
					}
				}
				
				int0 += "\n";
				
				// current step (if any
				PlanBody pb = im.getCurrentStep();
				if (pb == null)
					;//int0 += "no step, plan finished";
				else
					int0 += "step: "+im.getCurrentStep().toString();
				int0 += "\n";
			}
			intLog.info(int0);
		}
	}
	
	public void intentionSuspended(Intention i, String reason) {
	}
	
	
    public void intentionResumed(Intention i) {
	}

	@Override
	public void intentionDropped(Intention arg0) {
		
		// is intention null (can happen, don't know why)
		if (arg0 == null)
			return;
		
		// intention is finished, do nothing
		if (arg0.isFinished())
				if (!optDelibNoIntentionRestart)
				;//	this.ts.updateEvents(createDummyEvent());
		else {
			if (arg0.getIMs().size() > 0) {
				IntendedMeans im = arg0.getIMs().firstElement();
				String errStr =this.agName + " intention failed, trigger: "+im.getTrigger().toString();
				System.err.println(errStr);
				intLog.info(errStr);
			}
			else {
				// intention not dropped, but no intended means on stack (intention
				// is removed next rc)
				//System.err.println(this.agName + " intention ended: "+arg0.getId());
			}
						
		}
		
		if ((!optDelibNoIntentionRestart) && (this.cowDelib != null))			
			this.ts.updateEvents(createDummyEvent());
	}
	
	protected void runDelib() {
		if (this.cowDelib == null)
			return;
		
		Queue<Event> events = new LinkedList<Event>();
		events.add( this.createDummyEvent() );
		Event trigger  = null;
		try {
			trigger = cowDelib.reconsider(events, cowDesGen.getDesireState(), this.getTS());
		} catch (Exception e) {
			System.out.println("delib fehler: "+e);
			System.out.println(e.getStackTrace());
		}
		if (trigger != null) {
			ts.updateEvents(trigger);
		}
	}
	
	
	protected Event createDummyEvent() {
		Literal l = Literal.parseLiteral("dummyEvent");
		Trigger tr = new Trigger(TEOperator.add, TEType.belief, l);
		Event evnt = new Event(tr, Intention.EmptyInt);
		return evnt;
	}
	
	@Override
	public void abolish(Literal bel, Unifier un) throws RevisionFailedException {
        
		List<Literal> toDel = new ArrayList<Literal>();
        if (un == null) un = new Unifier();
        Iterator<Literal> il = getBB().getCandidateBeliefs(bel, un);
        if (il != null) {
            while (il.hasNext()) {
                Literal inBB = il.next();
                if (!inBB.isRule()) {
                    // need to clone unifier since it is changed in previous iteration
                    if (un.clone().unifiesNoUndo(bel, inBB)) {
                        toDel.add(inBB);
                    }
                }
            }
        }
           
        // support fast purge
        int delCount = 0;
        for (Literal l: toDel) {
        	if (!l.hasSource()) {
                l.addAnnot(BeliefBase.TSelf);
            }
        	cowRevision.revise_belief(null, l, null);
        	this.bb.remove(l);
        	++delCount;
        }
    }

	@Override
	public DesireComponent getCowDesires() {
		return this.cowDesGen;
	}
	
}
