package edu.udo.cs.ie.cowbots.agent;

import edu.udo.cs.ie.cowbots.bdi.*;
import edu.udo.cs.ie.cowbots.bdi.actsel.*;
import edu.udo.cs.ie.cowbots.bdi.brf.IEpistemicState;
import edu.udo.cs.ie.cowbots.bdi.delib.DeliberationComponent;
import edu.udo.cs.ie.cowbots.bdi.desgen.*;
import edu.udo.cs.ie.cowbots.logic.solver.*;
import edu.udo.cs.ie.cowbots.ui.*;

/**
 * 
 * basic interface that offers methods to access cowbot agent
 * specific features.
 * 
 * @author Thomas
 *
 */
public interface ICowbot {

	/**
	 * this method returns an instance to a cowbot ui
	 */
	public ICowbotUI	getUI();
	
	/**
	 * this method returns an instance to the action selection
	 */
	public IActionComponent	getActionSelection();
	
	/**
	 * getter to receive epistemic state
	 * @return instance to an agents epistemic state
	 */
	public IEpistemicState	getEpistemicState();
	
	/**
	 * getter to access dlv
	 * @return instance to dlv
	 */
	public DLV	getDLV();
	
	
	/**
	 * returns a path to an agent's local working
	 * directory. the path is absolute and ends
	 * with an "/".
	 * 
	 * @return path to an agent's temporary directory
	 */
	public String getTemporaryDirectory();
	
	
	/**
	 * returns a path to the agents log directory.
	 * any kind of logs could be written here. the
	 * directory path is absolute and ends with "/"
	 * 
	 * @return path to an agent's log directory
	 * 
	 */
	public String getLogDirectory();
	
	/**
	 * @return the cowDelib
	 */
	public DeliberationComponent getCowDelib();
	
	
	/**
	 * returns the desire generator of this cowbot agent
	 * 
	 * @return instance to a desire component
	 */
	public DesireComponent getCowDesires();
		
}
