package edu.udo.cs.ie.cowbots.arch;

import edu.udo.cs.ie.cowbots.arch.massim.MassimAgent;
import jason.architecture.AgArch;
import jason.asSemantics.ActionExec;
import jason.asSyntax.*;
import jason.JasonException;
import jason.mas2j.*;
import jason.runtime.*;
import java.util.*;
import java.util.logging.*;
import jason.asSemantics.Message;

import edu.udo.cs.ie.cowbots.agent.*;
import edu.udo.cs.ie.cowbots.launcher.CowbotsConfig;

/**
 * customization of the jason AgArch class to
 * meet requirements of our cowbot bdi implementation.
 * 
 * @author Thomas
 *
 */
public class CowbotArch extends AgArch {

	private MassimAdapter 		massimAdapter;
	private EnvironmentAdapter 	env;
	private ICowbot				cowAg;
	List<ActionResponse>		pendingActions;
	
	/**
	 * custom agent initialization.
	 */
	@Override
    public void initAg(String agClass, ClassParameters bbPars, String asSrc, Settings stts) throws JasonException {

		// initialize agent
		super.initAg(agClass, bbPars, asSrc, stts);
		
		// init bdi
		try {
			if (this.getTS().getAg() instanceof CowbotAgent) {
				cowAg =(CowbotAgent) this.getTS().getAg();
				((CowbotAgent) this.getTS().getAg() ).initBDI(stts, this.getAgName());
			}
		} catch (Exception e) {
			System.out.println("error!");
			System.out.println(e);
			for (int i = 0; i < e.getStackTrace().length; i++) {
				System.out.println(e.getStackTrace()[i]);
			}
			
			throw (new JasonException(e.toString()));
		}
		
		
		String user = stts.getUserParameter("user");
		System.out.println("massim user: "+user);
		String pass = stts.getUserParameter("pass");
		String host = stts.getUserParameter("host");
		if (host != null)
			host = host.replaceAll("\"", "");
		int port = Integer.parseInt(stts.getUserParameter("port"));
		
		// host override?
		CowbotsConfig cfg = CowbotsConfig.getInstance();
		if (cfg.getHost() != null)
			host = cfg.getHost();
		
		massimAdapter = new MassimAdapter(this, host, user, pass, port);
		env = massimAdapter;
		pendingActions = new LinkedList<ActionResponse>();
	}

	/**
	 * checks the environment adapter for new percepts
	 * and returns them. at the moment, this implementation
	 * simply loads all percepts in order of insertion
	 * from the environment adapter.
	 */
	@Override
	public List<Literal> perceive() {
		List<Literal> ret = null;
		
		// check the adapter if it has new data
		// environment adapter uses a thread-safe
		// queue, so we can simply dequeue here
		env.acquire();
		ret = env.getPercepts();
		env.release();
		
		if (ret != null) {
			
			// new percepts avail, update pending actions
			//int time0 = env.getWorldTime();
			Iterator<ActionResponse> arIter = pendingActions.iterator();
			while (arIter.hasNext()) {
				ActionResponse ar = arIter.next();
				ar.ae.setResult(true);
				ar.fb.add(ar.ae);
				arIter.remove();
			}
			
			return ret;
		} else
			return null;		
	}

	/**
	 * Send the action to the Massim Adapter
	 * While the action is being performed, the intention 
	 * that made the action execution choice should be suspended. 
	 * When the action is finished (successfully or not), it
	 * should be added in the feedback list so that the intention 
	 * can be resumed.
	 * In the case of massim, we just sends the action and 
	 * assume, that it was successfully performed by the environment
	 * when a new step begins.
	 * Possible actions are: skip, north, south, east, west,
	 * northwest, northeast, southwest, southeast
	 */
	@Override
	public void act(ActionExec action, List<ActionExec> feedback) {
		// send the Action String to Massim
		ICowbot cbot = (ICowbot) this.getTS().getAg();
		env.act( cbot.getActionSelection().selectActionForIntention(action.getActionTerm()).toString());
		this.pendingActions.add( new ActionResponse(action, feedback, env.getWorldTime()));
		System.out.println(this.getAgName()+" aktion gesendet: "+action.getActionTerm().toString()+", t="+env.getWorldTime());
		String im = action.getIntention().getIMs().toString();
		cowAg.getUI().updateIntentionState(im);
		
	}
	
	
	private class ActionResponse {
		public ActionExec ae;
		List<ActionExec> fb;
		int wt;
		
		ActionResponse(ActionExec ae, List<ActionExec> fb, int wt) {
			this.ae = ae;
			this.fb = fb;
			this.wt = wt;
		}
	}
	

}
