package edu.udo.cs.ie.cowbots.bdi.brf;

import java.util.List;
import java.util.Set;

import edu.udo.cs.ie.cowbots.logic.ELP;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import org.w3c.dom.*;

/**
 * basic interface for all kinds of knowledge
 * operators from the Cowbot BDI Model.
 * 
 * @author Thomas
 *
 */
public interface IKnowledgeOperator {

	/**
	 * default initialization routine, will be called by the
	 * revision engine with a configuration description in xml
	 *   
	 * @param config	XML Element describing operator setup
	 */
	public void init(Element config); 
	
	/**
	 * this method integrates information objects into the
	 * belief base. 
	 * 
	 * @param updates	new information objects
	 * @param bb		operator's belief base
	 * @return			true if something changed (false otherwise).
	 */
	public boolean update(List<InformationObject> updates, BeliefBase bb);
	
	
	/**
	 * method that might ist called by the revision engine
	 * for facts occuring in the reject set of an inference
	 * operator. each knowledge operator is free to process
	 * or ignore the call.
	 *  
	 * @param toDel	set of (head) literals to purge from the belief base
	 */
	public void deleteFacts(Set<ELPLiteral> toDel);
	
	/**
	 * method that is called after the revision (asp call and answer
	 * set clearance) to add/delete beliefs to the belief set. the
	 * behavior depends on the implemented knowledge operator semantics.
	 * 
	 * @param bs
	 * @param base
	 */
	public void processBeliefSet(BeliefSet bs, BeliefBase base); 
	
	
	/**
	 * method that returns an extended logical programm (or null)
	 * that is concatenated to the belief state, therefore
	 * bypassing any belief state construction.
	 *   
	 * @return	null or elp to add to the belief state
	 */
	public ELP getStateProgram();
	
	/**
	 * resets the knowledge operator, putting it back into
	 * a neutral state.
	 */
	public void reset();
}
