package edu.udo.cs.ie.cowbots.bdi.brf;

import java.util.*;


/**
 * this class models the update policy. an update policy
 * is used to assign incoming percepts to a knowledge
 * operator that stores the perept as an information
 * object inside the belief base.
 * 
 * @author Thomas
 *
 */
public class UpdatePolicy {
	
	protected 	Map<String, IKnowledgeOperator>  updmap;
	protected	IKnowledgeOperator defko;
	
	/**
	 * default constructor
	 * 
	 */
	public UpdatePolicy() {
		updmap = new HashMap<String, IKnowledgeOperator>();		
	}
	
	
	/**
	 * creates a new update policy with a default operator
	 * 
	 * @param ko	default knowledge operator
	 */
	public UpdatePolicy(IKnowledgeOperator ko) {
		defko = ko;
		updmap = new HashMap<String, IKnowledgeOperator>();
	}
	
		
		
	/**
	 * sets default operators for perception and self ports
	 * @param defPer	default perception operator
	 */
	public void setDefaults(IKnowledgeOperator defPer) {
		this.defko = defPer;
	}
	
	
	/**
	 * adds a knowledge operator to the policy. 
	 * @param o			operator to add
	 * @param cspl		comma separated predicate list
	 * @param config	(optional) operator configuration
	 */
	public void addOp(IKnowledgeOperator o, String cspl) {
		String preds[] = cspl.split(",");
		for (int i = 0; i < preds.length; i++) {
			String p = preds[i].trim();
			if (p.length() < 1)
				continue;
			this.updmap.put(p, o);
		}
	}
	

	/**
	 * returns a knowledge operator associated with pred,
	 * or the default knowledge operator.
	 * 
	 * @param pred	predicate looking for operator
	 * @return		knowledge operator for pred
	 * 
	 */
	public IKnowledgeOperator getOp(String pred) {
		IKnowledgeOperator ret = this.updmap.get(pred);
		if (ret == null)
			ret = defko;
		
		return ret;
	}
	
	/**
	 * debug output
	 */
	public String toString() {
		String ret = "update Policy Debug output\n";
		
		ret += "default: " + defko.getClass().getCanonicalName();
		
		ret +="\npredicates\n";
		
		Set<String> keys = updmap.keySet();
		
		for (String k : keys) {
			IKnowledgeOperator ko = updmap.get(k);
			ret += k + " : " + ko.getClass().getCanonicalName() + "\n";
		}
		return ret;
	}
}
