package edu.udo.cs.ie.cowbots.elpParser.dlp;


import edu.udo.cs.ie.cowbots.logic.ELP;
import java.io.*;
import java.util.ArrayList;
import java.util.List;



/**
 * this class extends from the class ELP and is used only 
 * by parse of disjunctive extended logic programs which
 * consist of a set of rules (and fact and constraints).
 * 
 * @author Wei
 * @see ELP
 */

public class DisjunctiveLogicProgram extends ELP {
	
	ArrayList<DLPRule> rules;
	// extra attributes for parsing
	private Vocabulary vocabulary;
	private DLPTermList terms;
	
	/**
	 * Default constructor
	 */
	public DisjunctiveLogicProgram() {
		
		rules = new ArrayList<DLPRule>();
		vocabulary = new Vocabulary();
		terms = new DLPTermList();
		
	}
	
	/**
	 * add a new rule to DLP.  
	 * @param r a rule {@link DLPRule}  
	 */	
	public void addDlpRule(DLPRule r) {
		rules.add(r);
	}
	
	/**
	 * view content of a DLP 
	 */
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		for(DLPRule r : rules) {
			sb.append(r.toString()+"\n");
		}
		
		return sb.toString();
	}
	
	/**
	 * size of rules in DLPs
	 * @return size {@link Integer}
	 */
	public int nRules() {
		return rules.size();
	}
	
	/**
	 * get i-rule 
	 * @param i index of literal {@link Integer}
	 * @return aimed literal {@link DLPLiteral}
	 */
	public DLPRule getRule(int i) {
		return rules.get(i);
	}
	
	/**
	 * get all rules in the program  
	 *  
	 * @return {@link DisjunctiveLogicProgram#rules}
	 */
	public ArrayList<DLPRule> getDlpRules() {
		return this.rules;
	}
	
	/**
	 * get Character and Predicate Dictionary for DLPs
	 *  
	 * @return {@link Vocabulary}
	 */
	public Vocabulary getVocabulary(){
		return vocabulary;
	}
	
	/**
	 * get Character and Predicate Dictionary for DLPs
	 *  
	 * @return {@link List}<{@link DLPTerm}>
	 */
	public void addTermList(List<DLPTerm> terms){
		this.terms.addTermList(terms);
	}
	
	
	/**
	 * @see ELP#getFacts() 
	 */
	public List<DLPLiteral> getDlpFacts() {
		ArrayList<DLPLiteral> ret = new ArrayList<DLPLiteral>();
		for (DLPRule r : rules) {
			if (!r.isFact())
				continue;
			
			int iI = r.nHead();
			for (int i = 0; i < iI; i++) {
				ret.add( r.getHead(i));
			}
		}
		return ret;
	}
	
	
	/**
	 * store DLP on disc
	 * 
	 * @param filename output file (should like be foo.dlp or bar.txt) {@link String}
	 */
	public void saveAs(String filename) {
		try {
			BufferedWriter w = new BufferedWriter(new FileWriter(filename));
			for (DLPRule r : rules) {
				w.write(r.toString());
				w.newLine();
			}
			w.flush();
			w.close();
		} catch (Exception e) {
			e.getMessage();
		}
	}
	

}
