package edu.udo.cs.ie.cowbots.env;

import jason.environment.*;
import java.util.*;
import java.io.*;

import javax.swing.*;
import java.awt.*;

import edu.udo.cs.ie.cowbots.launcher.CowbotsConfig;

/**
 * a customized jason environment used to display massim 2010
 * svg output. 
 *  
 * @author Thomas
 *
 */
public class MassimMonitor extends Environment {

	// massim log directory
	protected String massimLogDir = null;
	
	// svg output directory monitoring thread
	protected MassimDirUpdate mduThread = null;
	
	// the gui
	protected MassimPanel masPanel = null;
	protected JFrame masWindow = null;
	protected JLabel masInfo = null;

	/**
	 * environment initialization routine. we should setup all directory
	 * listeners and graphics output here.
	 * 
	 * @author Thomas
	 */
	@Override
	public void init(String args[]) {
		Map<String, String> cmd = new HashMap<String, String>();

		// is a local config present?
		CowbotsConfig lcfg = CowbotsConfig.getInstance();
		
		// parse commands
		for (String s : args) {
			String s2[] = s.split("=");
			if (s2.length == 2) {
				s2[0] = s2[0].trim();
				s2[1] = s2[1].trim();
			}
			cmd.put(s2[0], s2[1]);
		}

		String logdir = cmd.get("dir");
		
		// local config override?
		if (lcfg.getMassimPath() != null)
			logdir = lcfg.getMassimPath();

		if (logdir != null) {
			// try creating environment
			try {
				mduThread = new MassimDirUpdate(logdir);
				mduThread.start();
				
				masPanel = new MassimPanel();
				// masPanel.setDoubleBuffered(true);
				masPanel.setPreferredSize(new Dimension(640, 480));
				masPanel.setVisible(true);
	
				masInfo = new JLabel();
				masInfo.setText("listening");
	
				masWindow = new JFrame();
				masWindow.setTitle("Massim 2010 SVG Viewer");
				masWindow.add(masInfo, BorderLayout.SOUTH);
				masWindow.add(masPanel);
				masWindow.pack();
				masWindow.setVisible(true);
			} catch (Exception e) {
				// do nothing here, maybe we don't have
				// a gui at all
			}
		} else {
			System.out.println("env: error, no massim svg output directory specified");
		}
	}

	/**
	 * called by the infrastructure when the mas shuts down. we should now stop
	 * all directory listener and draw threads.
	 * 
	 * @author Thomas
	 */
	@Override
	public void stop() {
		if (this.mduThread != null) {
			this.mduThread.shutdown();
			try {
				this.mduThread.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * this class observes a massim 2010 svg output directory. it automatically
	 * switches to the latest directory and tries to take the path of the
	 * masSim-0.svg and masSim-X.svg (X is the current turn).
	 * 
	 * @author Thomas
	 * 
	 */
	class MassimDirUpdate extends Thread {

		protected boolean running = true;
		protected String listenDir = "";
		public MassimDirUpdate(String svgRoot) {
			this.listenDir = svgRoot;
		}

		/**
		 * thread routine
		 * 
		 * @author Thomas
		 * 
		 */
		@Override
		public void run() {
			try {
				while (this.isRunning()) {
					File f = new File(listenDir);
					File subdirs[] = f.listFiles( new FileFilter() {
						public boolean accept(File arg0) {
							return arg0.isDirectory();
						}
					});

					Arrays.sort( subdirs, new Comparator<File>() {
				      public int compare(File f1, File f2) {
				    	long t1 = f1.lastModified();
				    	long t2 = f2.lastModified();
				        return (t1 <= t2)? -1 : 1;
				      }
				    });
					
					// do we have a svg output directory?
					String svgDir = null;
					if (subdirs.length > 0)
						svgDir = subdirs[subdirs.length - 1].getName();

					// seems so, go on
					if (svgDir != null) {

						// find newest masSim-X.svg files
						f = new File(listenDir + svgDir + "/");
						String lm[] = f.list();
						int newest = -1;
						for (int i = 0; i < lm.length; i++) {
							String afile = lm[i];
							if (afile.startsWith("masSim-")) {
								String idx = afile.substring(7,
										afile.length() - 4);
								int int_idx = Integer.parseInt(idx);
								if (int_idx > newest)
									newest = int_idx;
							}
						}

						// found a file?
						if (newest >= 0) {
							String newMap = listenDir + svgDir + "/masSim-"+newest+".svg";
							String newBgnd = listenDir + svgDir
									+ "/masSim-0.svg";

							if (masInfo != null)
								masInfo.setText(newMap);
							if (masPanel != null) {
								masPanel.setSVG(newMap, newBgnd);
								masPanel.repaint();
							}

						}
					}
					// wait some time 
					Thread.sleep(1234);
				}
			} catch (Exception e) {
				System.err.println("env: directory listener crashed!");
			}
		}
		
		protected synchronized boolean isRunning() {
			return this.running;
		}
		
		public synchronized void shutdown() {
			this.running = false;
		}
	}
}
