package edu.udo.cs.ie.cowbots.internalActions.experimental2;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.internalActions.scout.astar;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Literal;
import jason.asSyntax.NumberTerm;
import jason.asSyntax.Term;

/**
 * 
 * @author dhoelzgen
 *
 */
public class exCreateFenceTasks extends DefaultInternalAction {

	/**
	 * Deprecated internal action used for testing the fence open mechanism. Replaced
	 * by exCreatePlan.
	 */
	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
	throws Exception {
		// Delete old tasks
		ts.getAg().abolish(Literal.parseLiteral("taskopenfence(_,_,_)"), un);
		
		// Agents position
		int aX = (int) ((NumberTerm) args[0]).solve();
		int aY = (int) ((NumberTerm) args[1]).solve();
		
		// Cows position
		int cX = (int) ((NumberTerm) args[2]).solve();
		int cY = (int) ((NumberTerm) args[3]).solve();
		
		// Targets position
		int tX = (int) ((NumberTerm) args[4]).solve();
		int tY = (int) ((NumberTerm) args[5]).solve();
		
		int time = (int) ((NumberTerm) args[6]).solve();
		
		// Init pathfinder
		astar pathfinder = new astar();
		pathfinder.experimental_cowsight = false;
		pathfinder.experimental_obstacle = false;
		pathfinder.experimental_newposition = true;
		
		ICowbot cowbot = (ICowbot) ts.getAg();
		GridMap map = cowbot.getEpistemicState().getMap();
		
		// Search the first fence between agent and cow
		String[] agToCow = pathfinder.getPath(map, aX, aY, cX, cY, false);
		int vX = aX;
		int vY = aY;
		
		boolean foundFence = false;
		
		for (String step : agToCow) {
			
			// Search for a fence
			if (map.getContent(vX, vY).getAtom().getTerm(0).contains("fence")) {
				ts.getAg().addBel(Literal.parseLiteral("taskopenfence("+vX+","+vY+","+time+")"));
				foundFence = true;
				break;
			}
			
			// Determine next position
			if (step.indexOf("north") != -1)
				vY -= 1;
			if (step.indexOf("south") != -1)
				vY += 1;
			if (step.indexOf("east") != -1)
				vX += 1;
			if (step.indexOf("west") != -1)
				vX -= 1;
		}
		
		if (!foundFence) {
			// Search the first fence between cow and target
			String[] cowToTarget = pathfinder.getPath(map, cX, cY, tX, tY, false);
			cX = aX;
			cY = aY;
			
			for (String step : cowToTarget) {
				
				// Search for a fence
				if (map.getContent(vX, vY).getAtom().getTerm(0).contains("fence")) {
					ts.getAg().addBel(Literal.parseLiteral("taskopenfence("+vX+","+vY+","+time+")"));
					break;
				}
				
				// Determine next position
				if (step.indexOf("north") != -1)
					vY -= 1;
				if (step.indexOf("south") != -1)
					vY += 1;
				if (step.indexOf("east") != -1)
					vX += 1;
				if (step.indexOf("west") != -1)
					vX -= 1;
			}
		}
		
		return true;
	}
}
