package edu.udo.cs.ie.cowbots.internalActions.experimental2;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.internalActions.scout.createWaypoints;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.util.DistanceMap;
import edu.udo.cs.ie.cowbots.util.GridMap;
import edu.udo.cs.ie.cowbots.util.PathTransformer;
import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.Message;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;
import jason.asSyntax.Atom;
import jason.asSyntax.Literal;
import jason.asSyntax.NumberTerm;
import jason.asSyntax.NumberTermImpl;
import jason.asSyntax.PredicateIndicator;
import jason.asSyntax.Term;

/**
 * 
 * @author dhoelzgen
 *
 */
public class exCreateWaypoints extends DefaultInternalAction {

	/**
	 * Generates the Waypoints for the ex2 Agents. Due to some strategy changes,
	 * there is no need to add other waypoints than the startpoint to the
	 * agents beliefs.
	 */
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
	throws Exception {
		// Delete old kwaypoint beliefs
		ts.getAg().abolish(Literal.parseLiteral("kwaypoint(_,_,_,_,_,_)"), un);

		int sX = (int) ((NumberTerm) args[0]).solve();
		int sY = (int) ((NumberTerm) args[1]).solve();
		int tX = (int) ((NumberTerm) args[2]).solve();
		int tY = (int) ((NumberTerm) args[3]).solve();

		// Init pathfinder
		ICowbot cowbot = (ICowbot) ts.getAg();
		GridMap map = cowbot.getEpistemicState().getMap();
		DistanceMap pathfinder = map.getDistanceMap(false);

		// Set target
		pathfinder.fill(tX, tY);

		// Set start position
		LinkedList<Point> path = pathfinder.getPath(sX, sY);

		String direction = "skip";

		if (path != null) {
			// first point on path is always the starting point
			// so we always have to use 2nd entry to get a direction
			// for moving.
			if (path.size() > 1) {
				direction = PathTransformer.getMove(path.get(0), path.get(1));
			}
		}

		Literal startWp = this.createLiteralWithPredicate("startwp", 0, sX, sY, "na", direction);
		
		// Add wp to the agents beliefs
		ts.getAg().addBel(startWp);

		return true;
	}

	/**
	 * Creates Waypoint of type pre, adds prefix k
	 * 
	 * @param pre
	 * @param stepNo
	 * @param vX
	 * @param vY
	 * @param lastDirection
	 * @param direction
	 * @return
	 */
	protected Literal createLiteralWithPredicate(String pre, int stepNo, int vX, int vY, String lastDirection, String direction) {
		return Literal.parseLiteral("kwaypoint(" + pre + "," + String.valueOf(stepNo) + ","
				+ String.valueOf(vX) + "," 
				+ String.valueOf(vY) + "," 
				+ lastDirection + "," + direction
				+ ")");
	}
}
