package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.bdi.brf.BeliefSet;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;


/**
 * Class for calculating the distance  between the own desired scoutDestination
 * and all scoutOptions and -Destinations of other Agents
 * 
 * @author bienek, jablkowski
 *
 */
public class checkScoutArea extends DefaultInternalAction {
	Point pointToCheck = new Point();
	List<ELPLiteral> scoutDestinations = new LinkedList<ELPLiteral>();
	List<ELPLiteral> scoutOptions = new LinkedList<ELPLiteral>();
	List<ELPLiteral> agentNumbers = new LinkedList<ELPLiteral>();
	String myName ;
	int delta = 5;
	ICowbot cowbot;
	
	BeliefSet bSet;
	boolean possible;
	
	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
			throws Exception {
		pointToCheck.x = (int) ((NumberTerm) args[0]).solve(); 
		pointToCheck.y = (int) ((NumberTerm) args[1]).solve(); 
		cowbot = (ICowbot) ts.getAg();
		bSet = cowbot.getEpistemicState().getBeliefSet();
		
		getScoutInfos();
		possible = checkConflicts();
		
		if(!possible)un.unifies(args[2], new NumberTermImpl(0));
		else un.unifies(args[2], new NumberTermImpl(1));
		//un.unifies(args[0], new StringTermImpl(""+possible));
		return true;
	}
	
	/**
	 * Utility function to retrieve relevant beliefs
	 */
	private void getScoutInfos()
	{
		for(ELPLiteral l : bSet.getBeliefs())
		{
			if(l.getAtom().getPredicate().equals("scoutDestination")) scoutDestinations.add(l);
			else if(l.getAtom().getPredicate().equals("scoutOption")) scoutOptions.add(l);
			else if(l.getAtom().getPredicate().equals("iam")) myName = l.getAtom().getTerm(0);
			else if(l.getAtom().getPredicate().equals("teamnumber")) agentNumbers.add(l);
		}
		
	}
	
	/**
	 * Utility function for calculating whether the scoutDestination is valid.
	 * Compares the distance to other scoutOptions and scoutDestinations with delta.
	 * 
	 * @return	result-boolean, true, if there are no conflicts, false else
	 *
	 */
	private boolean checkConflicts()
	{
		
		for(ELPLiteral l : scoutDestinations){
		if (!l.getAtom().getTerm(0).equals("self")){
			if(Math.abs(pointToCheck.x- Integer.parseInt(l.getAtom().getTerm(1)))<delta && Math.abs(pointToCheck.y- Integer.parseInt(l.getAtom().getTerm(2)))<delta) return false; 
		}
		}
		
		for(ELPLiteral l : scoutOptions){
			if (!l.getAtom().getTerm(0).equals("self")){
				
				if(Math.abs(pointToCheck.x- Integer.parseInt(l.getAtom().getTerm(1)))<delta && Math.abs(pointToCheck.y- Integer.parseInt(l.getAtom().getTerm(2)))<delta)
				{
					int numberofotherAgent=0;
				    int mynumber=0;
					for(ELPLiteral k: agentNumbers)
					{
						if(k.getAtom().getTerm(0).equals(l.getAtom().getTerm(0))) numberofotherAgent = Integer.parseInt(k.getAtom().getTerm(1));
						else if (k.getAtom().getTerm(0).equals(myName)) mynumber = Integer.parseInt(k.getAtom().getTerm(1));
					}
					if(numberofotherAgent < mynumber) return false;
					
				}
			}
		}
		
		return true;
	}
	private void print(String s)
	{
		System.out.println(s);
	}
}
