package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;

/**
 * Class for choosing the nearest Helper to a certain Point
 * 
 * @author bienek, jablkowski
 *
 */
public class getNearestHelper  extends DefaultInternalAction{

	ICowbot cowbot;
	
	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
			throws Exception {
		
	cowbot = (ICowbot) ts.getAg();
		
		
	int xcoord = (int) ((NumberTerm) args[0]).solve();
    int ycoord = (int) ((NumberTerm) args[1]).solve();
	String fenceopenername = args[2].toString();	
	String helpername =	getHelper(xcoord, ycoord, fenceopenername);
	
	un.unifies(args[3], new Atom(helpername));
    
    
	return true;
	}
	
	/**
	 * Utility function for calculating the distance between 2 points 
	 * and comparing it to a threshold delta
	 * 
	 * @param x	X-coordinate of the Point
	 * @param y	Y-coordinate of the Point
	 * @return	the name of the nearest helper
	 *
	 */
	private String getHelper(int x, int y, String fenceopenername){
		List<ELPLiteral> helperpositions = cowbot.getEpistemicState().getBeliefSet().getBeliefsByPredicate("helperison");
		
		if (helperpositions.size() != 0){
		int nearest = Integer.MAX_VALUE;
		String bestcandidate = helperpositions.get(0).getAtom().getTerm(0);
		
		for (ELPLiteral e: helperpositions){
		  int distance= calcDistance(e,x,y);
		  
		  if (distance< nearest & !e.getAtom().getTerm(0).equals(fenceopenername) ){ 
			  nearest = distance;
              bestcandidate = e.getAtom().getTerm(0);		   
		  }
		}
		
		return bestcandidate;
		}
		else return	cowbot.getEpistemicState().getBeliefSet().getBeliefsByPredicate("helper").get(0).getAtom().getTerm(2);
				
	}
	
	/**
	 * Utility function for calculating the distance between Agent and checkpoint
	 * @param e the ELPLiteral encoding the agents position.
	 * @param x	X-coordinate of the CP
	 * @param y	Y-coordinate of the CP
	 * @return	the distance
	 *
	 */
	private int calcDistance (ELPLiteral e, int x, int y){
		int diffX = Math.abs(x-Integer.parseInt(e.getAtom().getTerm(1)));
		int diffY = Math.abs(y-Integer.parseInt(e.getAtom().getTerm(2)));
		
		
		return diffX+diffY;
	}
	
}
