package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;

/**
 * Class for calculating the position to handle the switch
 * while passing a fence
 * 
 * @author bienek, jablkowski
 *
 */
public class getOpeningPosition extends DefaultInternalAction {

	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
			throws Exception {

		int wpxcoord = (int) ((NumberTerm) args[0]).solve();
		int wpycoord = (int) ((NumberTerm) args[1]).solve();
		int swxcoord = (int) ((NumberTerm) args[2]).solve();
		int swycoord = (int) ((NumberTerm) args[3]).solve();
		String direction = args[4].toString();
		int opencloseflag = (int) ((NumberTerm) args[5]).solve();

		Point p = calculatetarget(opencloseflag, wpxcoord, wpycoord, swxcoord,
				swycoord, direction);

		un.unifies(args[6], new NumberTermImpl(p.x));
		un.unifies(args[7], new NumberTermImpl(p.y));

		return true;
	}

	/**
	 * Utility function for calculating the target position
	 * 
	 * @param f	int-flag, value = 1, if the position for a "doorcloser" needs to be found,
	 * value = 0 for a dooropener
	 * @param x	X-coordinate of the WP needed to calculate the Position
	 * @param y	Y-coordinate of the WP needed to calculate the Position
	 * @param w X-coordinate of the switch
	 * @param u Y-coordinate of the switch
	 * @param direction the direction of the fence (vertical or horizontal)
	 * @return	Point encoding the coordinates
	 *
	 */
	private Point calculatetarget(int f, int x, int y, int w, int u,
			String direction) {

		if (f == 0) {
			if (direction.equals("vertical"))
				return new Point(x, u);
			else
				return new Point(w, y);
		} else {
			int relevantx = w;
			int relevanty = u;

			if (direction.equals("vertical")) {
				if (w - x < 0)
					relevantx = w - 1;
				else
					relevantx = w + 1;
			} else {
				if (u - y < 0)
					relevanty = u - 1;
				else
					relevanty = u + 1;
			}
			return new Point(relevantx, relevanty);
		}

	}

}
