package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;

/**This class implements an internal action which computes a candidate point for
*the scout, if the world is known (the predicate "worldknown must be set in the
*context of the ASL-Plan). Since there is no Weight for the Points and we dont want 
*any Point favored, we just choose a target at random.
*
* @author bienek, jablkowski
*/
public class getRandomScoutCandidate extends DefaultInternalAction {

	GridMap map;
	ICowbot cowbot;
	Point randomtarget;

	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
			throws Exception {
		cowbot = (ICowbot) ts.getAg();
		map = cowbot.getEpistemicState().getMap();
		randomtarget = calculateRandomTarget();
		un.unifies(args[0], new NumberTermImpl(randomtarget.x));
		un.unifies(args[1], new NumberTermImpl(randomtarget.y));

		return true;
	}

	/**
	 * Utility function for calculating a random target in the world, which is
	 * not a switch or an obstacle
	 * 
	 * @return	Point encoding the coordinates
	 */
	private Point calculateRandomTarget() {

		int Xcoordinate = (int) (Math.random() * map.width());
		int Ycoordinate = (int) (Math.random() * map.height());
		if(Xcoordinate == 0) Xcoordinate++;
		if(Ycoordinate == 0) Ycoordinate++;
		if(Xcoordinate == map.width()) Xcoordinate--;
		if(Ycoordinate == map.height()) Ycoordinate--;
		if (map.getContent(Xcoordinate, Ycoordinate).getAtom().getTerm(0).equals("obstacle") 
		 ||(map.getContent(Xcoordinate, Ycoordinate).getAtom().getTerm(0).equals("switch")))
		 return calculateRandomTarget();
		else return (new Point(Xcoordinate, Ycoordinate));

	}
}
