package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;

class DistanceComparator implements Comparator<WeightedPoint> {
	
	public int compare(WeightedPoint p1, WeightedPoint p2) {
	int distanceP1 = p1.getDistance();
	int distanceP2 = p2.getDistance();

	if (distanceP1 > distanceP2) return  1;
	else if (distanceP1 < distanceP2) return  -1;
	else return  0;
	
    }
}

class WeightComparator implements Comparator<WeightedPoint> {
	
	public int compare(WeightedPoint p1, WeightedPoint p2) {
		int weightP1 = p1.getWeight();
		int weightP2 = p2.getWeight();

		if (weightP1 < weightP2) return 1;
		else if (weightP1 > weightP2) return -1;
		else return 0; 
	}
}

// This class link points(candidates) to weight
class WeightedPoint {
	private Point point;
	private int weight;
	private int distance;

	public WeightedPoint(Point point, int weight) {
		this.point = point;
		this.weight = weight;
	}

	public Point getPoint() {
		return point;
	}

	public int getWeight() {
		return weight;
	}

	public void setDistance(int d) {
		distance = d;
	}

	public int getDistance() {
		return distance;
	}

}

/** This class implements an internal action which computes candidate points for
* the scout.
* 
* @author bienek, jablkowski
*/
public class getScoutCandidate extends DefaultInternalAction {

	GridMap map;
	int numberOfPoints;
	Point[] notWeightedCandidates;
	List<WeightedPoint> weightedCandidate;
	ICowbot cowbot;
	Point corralPos = new Point();
	int WorldKnownThreshold;
	
	

    
    
	@Override
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
			throws Exception {
		
		
		
		
		//threshold for worldknown to be set
		 WorldKnownThreshold = 15;
		
		numberOfPoints = 5;
		cowbot = (ICowbot) ts.getAg();
		map = cowbot.getEpistemicState().getMap();

		notWeightedCandidates = getRandomCandidates(numberOfPoints,ts);
		weightedCandidate = getWeightedCandidates(notWeightedCandidates);
		distToCorral(weightedCandidate);
		//printCandidte();
		orderCandidates(weightedCandidate);
		//printCandidte();
		 

		un.unifies(args[0], new NumberTermImpl(weightedCandidate.get(0).getPoint().x));
		un.unifies(args[1], new NumberTermImpl(weightedCandidate.get(0).getPoint().y));

		return true;
	}

	private void printCandidte()
	{
		for(int i=0;i<weightedCandidate.size();i++)
		 {
		 System.out.println("Weight : " +
		 weightedCandidate.get(i).getWeight() + ", Distance: " +weightedCandidate.get(i).getDistance() );
		 }
		System.out.println("----------------------------------");
	}
	
	/**
	 * Utility function to get a number of random points for candidates
	 * @param numberOfPoints the number of points generated
	 * @param ts the TransisionSystem
	 * @return	an array containing all the points
	 *
	 */
	private Point[] getRandomCandidates(int numberOfPoints, TransitionSystem ts) {
		Point[] tempPoints = new Point[numberOfPoints];

		List<Point> spots = new LinkedList<Point>();
		for (int y = 0; y < map.height(); y++) {
			for (int x = 0; x < map.width(); x++) {
				if (map.getContent(x, y).getAtom().getTerm(0).equals(
						"unexplored")) {
					spots.add(new Point(x, y));
				}
			}
		}
		
		//adds a new Belief "worldknown" into the BB of the Agent, if the list of "unexplored"s is smaller than the threshold
		if (spots.size() <= WorldKnownThreshold) {
			try{ts.getAg().addBel(Literal.parseLiteral("worldknown"));}
		    catch(Exception E) {}
		}
		//if the Map has lost all unexploreds, it should return a randomtarget instead
	    if (spots.size() == 0) {
	    	Point[] singleTempPoint = new Point[1];
	    	singleTempPoint[0] =calculateRandomTarget();
	    	return singleTempPoint;
	    }
		
		
		 
		for (int i = 0; i < numberOfPoints; i++) {

			int point = (int) (Math.random() * spots.size());
			tempPoints[i] = spots.get(point);
		}
		
		
		
		return tempPoints;
	}

	/**
	 * Utility function to apply a weight to each point
	 * @param p the array of points
	 * @return	the List of weighted points
	 */
	private List<WeightedPoint> getWeightedCandidates(Point[] p) {
		List<WeightedPoint> temp = new LinkedList<WeightedPoint>();
		for (int i = 0; i < p.length; i++) {
			int delta = 2;
			int startX = 0;
			int startY = 0;
			int endX = 0;
			int endY = 0;

			if (p[i].x - delta < 0)
				startX = 0;
			else
				startX = p[i].x - delta;

			if (p[i].x + delta > map.width())
				endX = map.width();
			else
				endX = p[i].x + delta;

			if (p[i].y - delta < 0)
				startY = 0;
			else
				startY = p[i].y - delta;

			if (p[i].y + delta > map.height())
				endY = map.height();
			else
				endY = p[i].y + delta;

			int tempWeight = 0;

			for (int j = startX; j < endX; j++) {
				for (int k = startY; k < endY; k++) {
					if (map.getContent(j, k).getAtom().getTerm(0).equals(
							"unexplored")) {
						tempWeight++;

					}
				}
			}
			temp.add(new WeightedPoint(p[i], tempWeight));
		}
		return temp;
	}

	
	private void distToCorral(List<WeightedPoint> weightedCandidate) {
		for (WeightedPoint wp : weightedCandidate) {
			wp.setDistance(Math.abs(wp.getPoint().x - corralPos.x)
					+ Math.abs(wp.getPoint().y - corralPos.y));
		}
	}
    
	/**
	 * Utility function to sort the targets
	 * @param weightedCandidate List of all weighted points
	 */
	private void orderCandidates(List<WeightedPoint> weightedCandidate) {
	    Collections.sort(weightedCandidate, new DistanceComparator());
		Collections.sort(weightedCandidate, new WeightComparator());

	}
 
	/**
	 * Utility function to get a Point if the Agent's beliefs no longer contain an unexplored cell
	 * 
	 * @return Point encoding the coordinates
	 */
	
	private Point calculateRandomTarget() {

		int Xcoordinate = (int) (Math.random() * map.width());
		int Ycoordinate = (int) (Math.random() * map.height());
		if(Xcoordinate == 0) Xcoordinate++;
		if(Ycoordinate == 0) Ycoordinate++;
		if(Xcoordinate == map.width()) Xcoordinate--;
		if(Ycoordinate == map.height()) Ycoordinate--;
		if (map.getContent(Xcoordinate, Ycoordinate).getAtom().getTerm(0).equals("obstacle") 
		 ||(map.getContent(Xcoordinate, Ycoordinate).getAtom().getTerm(0).equals("switch")))
		 return calculateRandomTarget();
		else return (new Point(Xcoordinate, Ycoordinate));

	}
}
