package edu.udo.cs.ie.cowbots.internalActions.scout;

import java.awt.Point;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.bdi.brf.BeliefSet;
import edu.udo.cs.ie.cowbots.internalActions.scout.astar.Node;
import edu.udo.cs.ie.cowbots.logic.ELPLiteral;
import edu.udo.cs.ie.cowbots.planer.makePlan;
import edu.udo.cs.ie.cowbots.util.GridMap;
import jason.*;
import jason.asSyntax.*;
import jason.asSemantics.*;

import jason.asSemantics.DefaultInternalAction;
import jason.asSemantics.IntendedMeans;
import jason.asSemantics.TransitionSystem;
import jason.asSemantics.Unifier;

/** This class implements an internal action which computes the partner in a 
 * 2-man-scout-team scenario. Its no longer used in the newest scout-version
 * 
 * @author bienek, jablkowski
 */
public class selectPartner extends DefaultInternalAction{
    
	ICowbot cowbot;
	BeliefSet bs;
	String partnerId; 
	String myId; 
	int myPrio=-1;
	 
	public Object execute(TransitionSystem ts, Unifier un, Term[] args)
	throws Exception {
		
		cowbot = (ICowbot) ts.getAg();
		bs = cowbot.getEpistemicState().getBeliefSet();
		myId = args[0].toString();
		partnerId= getPartnerId();
		
		
		un.unifies(args[1], new StringTermImpl(partnerId));
		
		return true;
	}

	/**
	 * Utility function to get partner's name
	 * @return	the agent's name
	 *
	 */
private String getPartnerId(){
	
	List<ELPLiteral> scoutlist =bs.getBeliefsByPredicate("iWantToScout");
	List<ELPLiteral> teamnumbers = bs.getBeliefsByPredicate("teamnumber");
	
	List<Partner> partners = new LinkedList<Partner>();
	
	if(myPrio <= -1){
		for(ELPLiteral l: teamnumbers){
			if (l.getAtom().getTerm(0).equals(myId)) myPrio = Integer.parseInt(l.getAtom().getTerm(1));
		}
		}
	
	for(ELPLiteral l: scoutlist){
		String scoutId = l.getAtom().getTerm(0);
		int scoutPrio = Integer.MAX_VALUE; 
		if (scoutId == "self") continue;
		for(ELPLiteral k: teamnumbers){
			if (k.getAtom().getTerm(0).equals(scoutId)) scoutPrio = Integer.parseInt(l.getAtom().getTerm(1));
		}
		partners.add(new Partner(scoutId,scoutPrio));
	}
	Collections.sort(partners);
	
	return partners.get(0).getId();
}
}

class Partner implements Comparable<Partner> {
	
	 private String id;
	 private int priority;
	
	public Partner(String id,int prio){
		this.id = id;
		this.priority = prio;
	}
	
	@Override
	public int compareTo(Partner partner) {

		if (this.priority < partner.priority)
			return -1;
		else if (this.priority > partner.priority)
			return 1;
		else
			return 0;

	}
	
	public String getId(){
		return id;
	}
}