package edu.udo.cs.ie.cowbots.planer;

import java.io.*;
import java.util.List;
import java.util.Set;

import jason.asSemantics.TransitionSystem;
import edu.udo.cs.ie.cowbots.agent.ICowbot;
import edu.udo.cs.ie.cowbots.elpParser.ElpParser;
import edu.udo.cs.ie.cowbots.elpParser.dlp.DisjunctiveLogicProgram;
import edu.udo.cs.ie.cowbots.logic.*;
import edu.udo.cs.ie.cowbots.logic.solver.DLV;

/*
 * This class uses DLV^K for higher planning
 * 
 * @author Elly
 * 
 */

public class HigherPlanner {

	DLV dlv;
	String myself = "myself";
	ELP elp;
	String[] tasks;
	//ElpParser parser;
	
	
	/*
	 * generates a .asl Plan by building a KProgram dynamically
	 * and using the Plan computed by DLV^K
	 * 
	 */
	public String generate(TransitionSystem ts, String[] tasks){
		String ret = "";

		ICowbot cb = (ICowbot) ts.getAg();
		myself = ts.getUserAgArch().getAgName();
		elp = cb.getEpistemicState().getBeliefsAsELP();
		dlv = cb.getDLV();
		this.tasks = tasks;
				
		//TODO GAAANZ wichtig: Parser zum Einlesen
		
		KProgram prog;
		
		//Name of the Plan to call
		String dynamicName = "dynamicHelp"; 
		
		// need other KProgram if there is a fence to open
		if (containesFenceOpenWP(elp)){
			dynamicName = "dynamicHelpWithFence";
			//TODO nearestAgent berechnen!
		}
		if (switchmissing(elp)){
			dynamicName = "dynamicHelpSearchSwitch";
			//TODO beide nearestAgents berechnen!
		}

		 prog = initNormalTaskAlloc(dynamicName, ts);
		
		// computes maxint
		List<ELPLiteral> facts = elp.getFacts();
		int wzx = 0, wzy =0, time = 0 ;
		for(ELPLiteral f : facts){
			if (f.getAtom().getPredicate().equals("worldsize")){
				wzx = Integer.valueOf(f.getAtom().getTerm(0));
				wzy = Integer.valueOf(f.getAtom().getTerm(1));
			}
			if (f.getAtom().getPredicate().equals("time")){
				time = Integer.valueOf(f.getAtom().getTerm(0));
			}
		}
		int wz = Math.max(Math.max(wzx, wzy) + 30, time);
		
		
		Set<ELPLiteral> planset;
		planset = dlv.computePlans(wz, ts, dynamicName);

		ret = buildPlan(planset);
				
		System.out.println(myself + " Ergebnis hheren Planens: " + ret);
		return ret;
		
	}

	/*
	 * checks whether the agent knows about a fence without switch
	 * 
	 */
	private boolean switchmissing(ELP elp) {
		for (ELPLiteral lit: elp.getFacts()){
			if (lit.getAtom().getPredicate().equals("noswitchknown")){
				return true;
			}
		}
		return false;
	}

	/*
	 * checks whether the actual beliefs of the agent 
	 * containes fence-Waypoints
	 */
	private boolean containesFenceOpenWP(ELP elp) {
		for (ELPLiteral lit: elp.getFacts()){
			if (lit.getAtom().getPredicate().equals("kwaypoint")&& 
					lit.getAtom().getTerm(1).startsWith("fenceopen") ){
				System.out.println(myself +" FenceOpenWP gefunden");
				return true;
			}
		}
		return false;
	}

	/*
	 * checks whether the actual beliefs of the agent 
	 * containes fence-Waypoints
	 */
	private boolean containesFenceCloseWP(ELP elp) {
		for (ELPLiteral lit: elp.getFacts()){
			if (lit.getAtom().getPredicate().equals("kwaypoint")&& 
					lit.getAtom().getTerm(1).startsWith("fenceclose") ){
				System.out.println(myself +" FenceCloseWP gefunden");
				return true;
			}
		}
		return false;
	}

/*
 * uses the output of DLV^K to build the .asl-Plan
 * 
 */
	
	private String buildPlan(Set<ELPLiteral> literals) {
		String ret = "";
		Boolean wpSent = false;
		ret+= "+!"+"dynamic"+" : true <-";
		ret += "?time(Q);";
		
		if(literals != null){
			int helpernumber =1;
			int numberofhelpers = literals.size();
			for (ELPLiteral lit: literals){
				// each literal has the form sendWP(helpername,wpType,wpNr)
				
				if (lit.getAtom().getPredicate().equals("sendWP")){
					wpSent = true;
					String whichWPType = lit.getAtom().getTerm(1);
					boolean first = true;
					//System.out.print(whichWPType + " gefunden ");
					//normal Situation, just move to WP
					if  (whichWPType.equals("startwp")){
						ret += "?kwaypoint(startwp,S,X,Y,F,D);";
						ret += "!sendWP(";
						ret += lit.getAtom().getTerm(0) +","+tasks[1] +",";
						ret += helpernumber++ + ","+ numberofhelpers +");";
//						ret += ".send(";
//						ret += lit.getAtom().getTerm(0);
//						//dynamisch machen
//						ret += ",tell,waypoint(startwp,drivencow,"+tasks[1] +",S,X,Y,F,D,";
//						ret += helpernumber++ +"," +numberofhelpers+ ",Q));";
					}
					//formation is formed, now move to next WP
					//TODO if next wp is not default!
					//TODO die reagieren mom nur auf startwp! daher hier nicht defaultwp!
					if (whichWPType.equals("defaultwp")){
						ret+= ".print(Def);?kwaypoint(defaultwp,S1,X1,Y1,F1,D1);";
						ret += ".send(";
						ret += lit.getAtom().getTerm(0);
						ret += ",tell,waypoint(startwp,drivencow,"+tasks[1] +",S1,X1,Y1,F1,D1,";
						ret += helpernumber++ +"," +numberofhelpers+ ",Q));";
					}
				}
				//Fence detected and should be opened
				if (lit.getAtom().getPredicate().equals("opengate")){
					ret += 	".findall(Agent, helper(_,Agent, _), HelperList );" +
							"!initiateAgents(HelperList);" +
							"!dooropeningOutside(HelperList);" +
							"-+whiledooropening(1);";
				}
				//Gate is open, now walk though, pretending, that startwp is behind the gate
				if (lit.getAtom().getPredicate().equals("walkthrough")){
					// walking through a normal fence
					if (containesFenceCloseWP(elp)){
						ret += 	"?kwaypoint(fenceclosewp,S3,X3,Y3,F3,D3);" +
						"-+whiledooropening(2);" ;
					} else {
						// driving cows into corral
						ret += 	"?kwaypoint(fenceopenwp,S2,X2,Y2,F2,D2);" +
						"-+whiledooropening(2);" ;
					}
					
				}
				//fencewp is reached, try to reach next wp if not all agents have passed
				if (lit.getAtom().getPredicate().equals("walkon")){
					ret += 	"?kwaypoint(fenceopenwp,_,X4,Y4,F4,D4);" +
							//"-kwaypoint(startwp,_,_,_,_,_);" +
							//"+kwaypoint(startwp,0,X4,Y4,F4,D4);" +
							"-+whiledooropening(3);" +
							".findall(Agent, helper(_,Agent, _), HelperList );" +
							"!initiateAgents(HelperList);" +
							".print(HelperList);" +
							"?currentFence(Fen,Xf,Yf);" +
							".print(Fen,Xf,Yf);" +
							".wait(100);" +
							"!askIfPassed(HelperList, Fen, Xf, Yf, D4, 1);" +
							"-+isPassed(Fen,Xf,Yf,D4);" +
							"!fencePassed;" +
							"-isPassed(Fen,Xf,Yf,D4);" +
							"!checkIfPassed;" +
							"!dooropeningInside(HelperList);" ;
				}
					
			}
		}
		
		if (wpSent){
			ret += "?waypointCounter(N);";
	//		ret += "-waypointCounter(N);";
			ret += "-+waypointCounter(N+1);";
		}
		

		//replace last ";" by "."
		int last = ret.length()-1;
		ret = ret.substring(0,last) + ".";
				
		//Ziel: .send(alice,tell,waypoint(startwp,drivencow,C,S,X,Y,F,D,1,5,Q));
		return ret;
	}
 

	/*
	 * Initializes the static background knowledge of a K Program
	 * using the beliefs of the agent
	 * 
	 */
	private KProgram initBel(TransitionSystem ts){
		KProgram prog = new KProgram();
		ICowbot cb = (ICowbot) ts.getAg();
		ELP elp = cb.getEpistemicState().getBeliefsAsELP();
				
		for(ELPRule r: elp.getRules()){
			prog.addKnowledge(r);
		}
		return prog;
	}
	
	
	/*
	 * Normally, the leader wants to send the startWP to each helper.
	 * This method creates a KProgram which can decide to send WPs to the helpers
	 * 
	 */
	private KProgram initNormalTaskAlloc(String saveName, TransitionSystem ts) {
		KProgram prog = initBel(ts);
				
		ELPRule rule = new ELPRule();
		rule.addHead(new ELPAtom("wpType", "startwp"));
		prog.addKnowledge(rule);

		rule = new ELPRule();
		rule.addHead(new ELPAtom("wpType", "fenceopenwp"));
		prog.addKnowledge(rule);
		
		rule = new ELPRule();
		rule.addHead(new ELPAtom("wpType", "defaultwp"));
		prog.addKnowledge(rule);
		
		//TODO Hilfskonstrukt, da "false" ein Schlsselwort ist und Fehler schmeit
		rule = new ELPRule();
		rule.addHead(new ELPAtom("falsch", "false"));
		prog.addKnowledge(rule);
		
		
////TODO just for testing		
//		rule = new ELPRule();
//		rule.addHead(new ELPAtom("wpType", "searchswitchwp"));
//		prog.addKnowledge(rule);
//
//// TODO just for testing
//		rule = new ELPRule();
//		rule.addHead(new ELPAtom("wpType", "switchwp"));
//		prog.addKnowledge(rule);
//
////TODO irgendwie mssen die wingAgents in die BB!		
//		rule = new ELPRule();
//		rule.addHead(new ELPAtom("wingAgent","alice"));
//		prog.addKnowledge(rule);
//		rule = new ELPRule();
//		rule.addHead(new ELPAtom("wingAgent","dave"));
//		prog.addKnowledge(rule);
//		
//		rule = new ELPRule();
//		rule.addHead(new ELPAtom("myhelper(X) :- helper(_,Me,X,_), iam(Me)"));
//		prog.addKnowledge(rule);
				
		
		
//		//Todo PARSER!!!!!!!
//		String kprogtext = "actions: " +
//			"sendWP(Ag,Type,Nr) requires agent(Ag), #int(Nr), wpType(Type)." +
//			"failure1. failure2."+
//			" fluents: is_on(A,B) requires #int(A), #int(B). " +
//			"busy(Ag) requires agent(Ag)." + "fenceopen. failurestate." +
//			" always: " +
//			"executable sendWP(Ag,startwp,N) if helper(_,_,Ag,_), not busy(Ag)," +
//			" kwaypoint(self,startwp,N,_,_,_,_).caused is_on(A,B) " +
//			"if kwaypoint(self,startwp,N,A,B,_,_) after sendWP(Ag,startwp,N)." +
//			"executable sendWP(Ag,fenceopenwp,N) if helper(_,_,Ag,_), not busy(Ag), " +
//			"kwaypoint(self,fenceopenwp,N,_,_,_,_)." +
//			"caused fenceopen if kwaypoint(self,fenceopenwp,N,A,B,_,_) " +
//			"after sendWP(Ag,fenceopenwp,N). " +
//			"caused busy(Ag) after sendWP(Ag,T,N). " +
//			"caused false if Ag1<>Ag2 after sendWP(Ag1,fenceopenwp,N), " +
//			"sendWP(Ag2,fenceopenwp,N). " +
//			"executable failure1 if not sendWP(Ag,T,N), " +
//			"helper(_,_,Ag,_),kwaypoint(_,T,N,_,_,_,_)." +
//			"caused failurestate after failure1." +
//			"executable failure2 if failurestate." +
//			"caused is_on(A,B) if #int(A), #int(B) after failure2." +
//			"caused false after sendWP(_,_,_), failure1." +
//			"initially:" +
//			"caused -busy(X) if helper(_,Me,X,_), iam(Me)." +
//			"securePlan." +
//			"goal: busy(alice), busy(bob), busy(charlie), busy(dave) ? (1)";
		
	
		prog.saveKnowledge("temp/"+myself+"/"+ saveName);
		//prog.testsave("temp/"+myself+"/"+ saveName, kprogtext);
		
		return prog;
	}
		

	
	
}
