package edu.udo.cs.ie.cowbots.util;

import java.util.*;
import java.awt.Point;

/**
 * 
 * auxiliary class that provide static methods
 * to transcode a path on a gridmap between
 * point based and textual direction based
 * representation
 * 
 * @author Thomas
 *
 */
public class PathTransformer {

	// some direction encodings
	static String[][] pEnc = { {"skip","north","south"}, // dx = 0
		{"west","northwest","southwest"}, // dx = 1
		{"east","northeast","southeast"} // dx = 2
	};
	
	/**
	 * this method returns the x-component of
	 * a direction dir
	 * 
	 * @param dir	a* direction
	 * @return		-1,0 or 1
	 */
	public static int getDirectionX(String dir) {
		if (dir == null)
			return 0;
		if (dir.equalsIgnoreCase("west"))
			return -1;
		if (dir.equalsIgnoreCase("northwest"))
			return -1;
		if (dir.equalsIgnoreCase("southwest"))
			return -1;
		if (dir.equalsIgnoreCase("east"))
			return +1;
		if (dir.equalsIgnoreCase("northeast"))
			return +1;
		if (dir.equalsIgnoreCase("southeast"))
			return +1;		
		return 0;
	}
	
	/**
	 * this method returns the y-component of
	 * a direction dir
	 * 
	 * @param dir	a* direction
	 * @return		-1,0 or 1
	 */
	public static int getDirectionY(String dir) {
		if (dir == null)
			return 0;
		if (dir.equalsIgnoreCase("north"))
			return -1;
		if (dir.equalsIgnoreCase("northwest"))
			return -1;
		if (dir.equalsIgnoreCase("southwest"))
			return +1;
		if (dir.equalsIgnoreCase("south"))
			return +1;
		if (dir.equalsIgnoreCase("northeast"))
			return -1;
		if (dir.equalsIgnoreCase("southeast"))
			return +1;		
		return 0;
	}
	
	/**
	 * this method returns the next point from
	 * a starting point and an a* direction
	 * 
	 * @param p		starting point
	 * @param dir	direction (north, northwest, etc..)
	 * @return		point at p + dir
	 */
	public static Point getNextPoint(Point p, String dir) {
		if (dir == null)
			return p;
	
		return new Point(p.x + getDirectionX(dir), p.y + getDirectionY(dir));
	}
	
	/**
	 * this method converts an astar path from a starting point
	 * into a list of points (absolute cell coordinates)
	 * @param start		starting point
	 * @param astarpath	a* caluclated path
	 * @return	path as point-list represenation
	 */
	public static List<Point> getPathPoints(Point start, String[] astarpath) {
		LinkedList<Point> ret = new LinkedList<Point>();
		
		ret.add(start);
		for (String dir : astarpath) {
			Point p = getNextPoint(start,dir);
			ret.add( p );
			start = p;
		}
		
		return ret;
	}
	
	/**
	 * this method returns a path in string based representation
	 * from a path in point based representation
	 * 
	 * @param path	path in point representation
	 * @return path in string representation
	 */
	public static String[] getPath(List<Point> path) {
		if (path == null)
			return null;
		
		String[] ret = null;
		if (path.size() <= 1) {
			ret = new String[1];
			ret[0] = "skip";
		} else {
		
			ret = new String[path.size()-1];
			int i = 0;
			
			Iterator<Point> pIter = path.iterator();
			Point p0 = pIter.next();
			
			while (pIter.hasNext()) {
				Point p1 = pIter.next();
				ret[i++] = getMove(p0,p1);
				p0=p1;		
			}
		}
		
		return ret;
	}
	
	/**
	 * returns the movement command equal to the direction
	 * represented by two points
	 * 
	 * @param p0	start point
	 * @param p1	end point
	 * @return	a direction like north, east, southwest, or skip
	 */
	public static String getMove(Point p0, Point p1) {
		int dx = 0;
		int dy = 0;
		if (p0.x < p1.x) // go east
			dx = 2;
		if (p0.x > p1.x) // go west
			dx = 1;
		
		if (p0.y > p1.y) // go north
			dy = 1;
		if (p0.y < p1.y) // go south
			dy = 2;
		
		return pEnc[dx][dy]; // encode [dx][dy]
	}
}
